package javax.imageio.metadata;

import java.util.Map;
import com.sun.imageio.plugins.common.StandardMetadataFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Locale;
import javax.imageio.ImageTypeSpecifier;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.HashMap;

public abstract class IIOMetadataFormatImpl implements IIOMetadataFormat
{
    public static final String standardMetadataFormatName = "javax_imageio_1.0";
    private static IIOMetadataFormat standardFormat;
    private String resourceBaseName;
    private String rootName;
    private HashMap elementMap;
    
    public IIOMetadataFormatImpl(String key, int childPolicy) {
        this.resourceBaseName = this.getClass().getName() + "Resources";
        this.elementMap = new HashMap();
        if (key == null) {
            throw new IllegalArgumentException("rootName == null!");
        }
        if (childPolicy < 0 || childPolicy > 5 || childPolicy == 5) {
            throw new IllegalArgumentException("Invalid value for childPolicy!");
        }
        this.rootName = key;
        IIOMetadataFormatImpl.Element value = new IIOMetadataFormatImpl.Element();
        value.elementName = key;
        value.childPolicy = childPolicy;
        this.elementMap.put(key, value);
        return;
    }
    
    public IIOMetadataFormatImpl(String key, int minChildren, int maxChildren) {
        this.resourceBaseName = this.getClass().getName() + "Resources";
        this.elementMap = new HashMap();
        if (key == null) {
            throw new IllegalArgumentException("rootName == null!");
        }
        if (minChildren < 0) {
            throw new IllegalArgumentException("minChildren < 0!");
        }
        if (minChildren > maxChildren) {
            throw new IllegalArgumentException("minChildren > maxChildren!");
        }
        IIOMetadataFormatImpl.Element value = new IIOMetadataFormatImpl.Element();
        value.elementName = key;
        value.childPolicy = 5;
        value.minChildren = minChildren;
        value.maxChildren = maxChildren;
        this.rootName = key;
        this.elementMap.put(key, value);
        return;
    }
    
    protected void setResourceBaseName(String resourceBaseName) {
        if (resourceBaseName == null) {
            throw new IllegalArgumentException("resourceBaseName == null!");
        }
        this.resourceBaseName = resourceBaseName;
        return;
    }
    
    protected String getResourceBaseName() {
        return this.resourceBaseName;
    }
    
    private IIOMetadataFormatImpl.Element getElement(String s, boolean b) {
        if (b && s == null) {
            throw new IllegalArgumentException("element name is null!");
        }
        IIOMetadataFormatImpl.Element element = (IIOMetadataFormatImpl.Element)this.elementMap.get(s);
        if (b && element == null) {
            throw new IllegalArgumentException("No such element: " + s);
        }
        return element;
    }
    
    private IIOMetadataFormatImpl.Element getElement(String s) {
        return this.getElement(s, true);
    }
    
    private IIOMetadataFormatImpl.Attribute getAttribute(String s, String str) {
        IIOMetadataFormatImpl.Attribute attribute = (IIOMetadataFormatImpl.Attribute)this.getElement(s).attrMap.get(str);
        if (attribute == null) {
            throw new IllegalArgumentException("No such attribute \"" + str + "\"!");
        }
        return attribute;
    }
    
    protected void addElement(String s, String s2, int childPolicy) {
        IIOMetadataFormatImpl.Element element = this.getElement(s2);
        if (childPolicy < 0 || childPolicy > 5 || childPolicy == 5) {
            throw new IllegalArgumentException("Invalid value for childPolicy!");
        }
        IIOMetadataFormatImpl.Element value = new IIOMetadataFormatImpl.Element();
        value.elementName = s;
        value.childPolicy = childPolicy;
        element.childList.add(s);
        value.parentList.add(s2);
        this.elementMap.put(s, value);
        return;
    }
    
    protected void addElement(String s, String s2, int minChildren, int maxChildren) {
        IIOMetadataFormatImpl.Element element = this.getElement(s2);
        if (minChildren < 0) {
            throw new IllegalArgumentException("minChildren < 0!");
        }
        if (minChildren > maxChildren) {
            throw new IllegalArgumentException("minChildren > maxChildren!");
        }
        IIOMetadataFormatImpl.Element value = new IIOMetadataFormatImpl.Element();
        value.elementName = s;
        value.childPolicy = 5;
        value.minChildren = minChildren;
        value.maxChildren = maxChildren;
        element.childList.add(s);
        value.parentList.add(s2);
        this.elementMap.put(s, value);
        return;
    }
    
    protected void addChildElement(String s, String s2) {
        IIOMetadataFormatImpl.Element element = this.getElement(s2);
        IIOMetadataFormatImpl.Element element2 = this.getElement(s);
        element.childList.add(s);
        element2.parentList.add(s2);
    }
    
    protected void removeElement(String key) {
        IIOMetadataFormatImpl.Element element = this.getElement(key, false);
        if (element != null) {
            Iterator<Object> iterator = element.parentList.iterator();
            while (iterator.hasNext()) {
                IIOMetadataFormatImpl.Element element2 = this.getElement((String)iterator.next(), false);
                if (element2 != null) {
                    element2.childList.remove(key);
                }
                else {
                    continue;
                }
            }
            this.elementMap.remove(key);
        }
    }
    
    protected void addAttribute(String s, String attrName, int dataType, boolean required, String defaultValue) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (attrName == null) {
            throw new IllegalArgumentException("attrName == null!");
        }
        if (dataType < 0 || dataType > 4) {
            throw new IllegalArgumentException("Invalid value for dataType!");
        }
        IIOMetadataFormatImpl.Attribute attribute = new IIOMetadataFormatImpl.Attribute();
        attribute.attrName = attrName;
        attribute.valueType = 1;
        attribute.dataType = dataType;
        attribute.required = required;
        attribute.defaultValue = defaultValue;
        element.attrList.add(attrName);
        element.attrMap.put(attrName, attribute);
        return;
    }
    
    protected void addAttribute(String s, String attrName, int dataType, boolean required, String defaultValue, List<String> enumeratedValues) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (attrName == null) {
            throw new IllegalArgumentException("attrName == null!");
        }
        if (dataType < 0 || dataType > 4) {
            throw new IllegalArgumentException("Invalid value for dataType!");
        }
        if (enumeratedValues == null) {
            throw new IllegalArgumentException("enumeratedValues == null!");
        }
        if (enumeratedValues.size() == 0) {
            throw new IllegalArgumentException("enumeratedValues is empty!");
        }
        for (Object next : enumeratedValues) {
            if (next == null) {
                throw new IllegalArgumentException("enumeratedValues contains a null!");
            }
            if (!(next instanceof String)) {
                throw new IllegalArgumentException("enumeratedValues contains a non-String value!");
            }
            continue;
        }
        IIOMetadataFormatImpl.Attribute o = new IIOMetadataFormatImpl.Attribute();
        o.attrName = attrName;
        o.valueType = 16;
        o.dataType = dataType;
        o.required = required;
        o.defaultValue = defaultValue;
        o.enumeratedValues = enumeratedValues;
        element.attrList.add(attrName);
        element.attrMap.put(attrName, o);
        return;
    }
    
    protected void addAttribute(String s, String attrName, int dataType, boolean required, String defaultValue, String minValue, String maxValue, boolean b, boolean b2) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (attrName == null) {
            throw new IllegalArgumentException("attrName == null!");
        }
        if (dataType < 0 || dataType > 4) {
            throw new IllegalArgumentException("Invalid value for dataType!");
        }
        IIOMetadataFormatImpl.Attribute attribute = new IIOMetadataFormatImpl.Attribute();
        attribute.attrName = attrName;
        attribute.valueType = 2;
        if (b) {
            attribute.valueType |= 0x4;
        }
        if (b2) {
            attribute.valueType |= 0x8;
        }
        attribute.dataType = dataType;
        attribute.required = required;
        attribute.defaultValue = defaultValue;
        attribute.minValue = minValue;
        attribute.maxValue = maxValue;
        element.attrList.add(attrName);
        element.attrMap.put(attrName, attribute);
        return;
    }
    
    protected void addAttribute(String s, String attrName, int dataType, boolean required, int listMinLength, int listMaxLength) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (attrName == null) {
            throw new IllegalArgumentException("attrName == null!");
        }
        if (dataType < 0 || dataType > 4) {
            throw new IllegalArgumentException("Invalid value for dataType!");
        }
        if (listMinLength < 0 || listMinLength > listMaxLength) {
            throw new IllegalArgumentException("Invalid list bounds!");
        }
        IIOMetadataFormatImpl.Attribute attribute = new IIOMetadataFormatImpl.Attribute();
        attribute.attrName = attrName;
        attribute.valueType = 32;
        attribute.dataType = dataType;
        attribute.required = required;
        attribute.listMinLength = listMinLength;
        attribute.listMaxLength = listMaxLength;
        element.attrList.add(attrName);
        element.attrMap.put(attrName, attribute);
        return;
    }
    
    protected void addBooleanAttribute(String s, String s2, boolean b, boolean b2) {
        ArrayList<String> list = new ArrayList();
        list.add("TRUE");
        list.add("FALSE");
        String s3 = null;
        if (b) {
            s3 = (b2 ? "TRUE" : "FALSE");
        }
        this.addAttribute(s, s2, 1, true, s3, list);
    }
    
    protected void removeAttribute(String s, String s2) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        element.attrList.remove(s2);
        element.attrMap.remove(s2);
    }
    
    protected <T> void addObjectValue(String s, Class<T> classType, boolean b, T defaultValue) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        IIOMetadataFormatImpl.ObjectValue objectValue = new IIOMetadataFormatImpl.ObjectValue();
        objectValue.valueType = 1;
        objectValue.classType = classType;
        objectValue.defaultValue = defaultValue;
        element.objectValue = objectValue;
    }
    
    protected <T> void addObjectValue(String s, Class<T> classType, boolean b, T defaultValue, List<? extends T> enumeratedValues) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (enumeratedValues == null) {
            throw new IllegalArgumentException("enumeratedValues == null!");
        }
        if (enumeratedValues.size() == 0) {
            throw new IllegalArgumentException("enumeratedValues is empty!");
        }
        for (Object next : enumeratedValues) {
            if (next == null) {
                throw new IllegalArgumentException("enumeratedValues contains a null!");
            }
            if (!classType.isInstance(next)) {
                throw new IllegalArgumentException("enumeratedValues contains a value not of class classType!");
            }
            continue;
        }
        IIOMetadataFormatImpl.ObjectValue objectValue = new IIOMetadataFormatImpl.ObjectValue();
        objectValue.valueType = 16;
        objectValue.classType = classType;
        objectValue.defaultValue = defaultValue;
        objectValue.enumeratedValues = enumeratedValues;
        element.objectValue = objectValue;
        return;
    }
    
    protected <T extends Object & Comparable<? super T>> void addObjectValue(String s, Class<T> classType, T defaultValue, Comparable<? super T> minValue, Comparable<? super T> maxValue, boolean b, boolean b2) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        IIOMetadataFormatImpl.ObjectValue objectValue = new IIOMetadataFormatImpl.ObjectValue();
        objectValue.valueType = 2;
        if (b) {
            objectValue.valueType |= 0x4;
        }
        if (b2) {
            objectValue.valueType |= 0x8;
        }
        objectValue.classType = classType;
        objectValue.defaultValue = defaultValue;
        objectValue.minValue = minValue;
        objectValue.maxValue = maxValue;
        element.objectValue = objectValue;
    }
    
    protected void addObjectValue(String s, Class<?> classType, int arrayMinLength, int arrayMaxLength) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        IIOMetadataFormatImpl.ObjectValue objectValue = new IIOMetadataFormatImpl.ObjectValue();
        objectValue.valueType = 32;
        objectValue.classType = classType;
        objectValue.arrayMinLength = arrayMinLength;
        objectValue.arrayMaxLength = arrayMaxLength;
        element.objectValue = objectValue;
    }
    
    protected void removeObjectValue(String s) {
        this.getElement(s).objectValue = null;
    }
    
    @Override
    public String getRootName() {
        return this.rootName;
    }
    
    @Override
    public abstract boolean canNodeAppear(String p0, ImageTypeSpecifier p1);
    
    @Override
    public int getElementMinChildren(String s) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (element.childPolicy != 5) {
            throw new IllegalArgumentException("Child policy not CHILD_POLICY_REPEAT!");
        }
        return element.minChildren;
    }
    
    @Override
    public int getElementMaxChildren(String s) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (element.childPolicy != 5) {
            throw new IllegalArgumentException("Child policy not CHILD_POLICY_REPEAT!");
        }
        return element.maxChildren;
    }
    
    private String getResource(String key, Locale default1) {
        if (default1 == null) {
            default1 = Locale.getDefault();
        }
        ClassLoader loader = (ClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return Thread.currentThread().getContextClassLoader();
            }
        });
        ResourceBundle resourceBundle;
        try {
            resourceBundle = ResourceBundle.getBundle(this.resourceBaseName, default1, loader);
        }
        catch (MissingResourceException ex) {
            try {
                resourceBundle = ResourceBundle.getBundle(this.resourceBaseName, default1);
            }
            catch (MissingResourceException ex2) {
                return null;
            }
        }
        try {
            return resourceBundle.getString(key);
        }
        catch (MissingResourceException ex3) {
            return null;
        }
    }
    
    @Override
    public String getElementDescription(String s, Locale locale) {
        this.getElement(s);
        return this.getResource(s, locale);
    }
    
    @Override
    public int getChildPolicy(String s) {
        return this.getElement(s).childPolicy;
    }
    
    @Override
    public String[] getChildNames(String s) {
        IIOMetadataFormatImpl.Element element = this.getElement(s);
        if (element.childPolicy == 0) {
            return null;
        }
        return (String[])(String[])element.childList.toArray(new String[0]);
    }
    
    @Override
    public String[] getAttributeNames(String s) {
        List<Object> attrList = this.getElement(s).attrList;
        return (String[])(String[])attrList.toArray(new String[attrList.size()]);
    }
    
    @Override
    public int getAttributeValueType(String s, String s2) {
        return this.getAttribute(s, s2).valueType;
    }
    
    @Override
    public int getAttributeDataType(String s, String s2) {
        return this.getAttribute(s, s2).dataType;
    }
    
    @Override
    public boolean isAttributeRequired(String s, String s2) {
        return this.getAttribute(s, s2).required;
    }
    
    @Override
    public String getAttributeDefaultValue(String s, String s2) {
        return this.getAttribute(s, s2).defaultValue;
    }
    
    @Override
    public String[] getAttributeEnumerations(String s, String s2) {
        IIOMetadataFormatImpl.Attribute attribute = this.getAttribute(s, s2);
        if (attribute.valueType != 16) {
            throw new IllegalArgumentException("Attribute not an enumeration!");
        }
        List<Object> enumeratedValues = attribute.enumeratedValues;
        enumeratedValues.iterator();
        return (String[])(String[])enumeratedValues.toArray(new String[enumeratedValues.size()]);
    }
    
    @Override
    public String getAttributeMinValue(String s, String s2) {
        IIOMetadataFormatImpl.Attribute attribute = this.getAttribute(s, s2);
        if (attribute.valueType != 2 && attribute.valueType != 6 && attribute.valueType != 10 && attribute.valueType != 14) {
            throw new IllegalArgumentException("Attribute not a range!");
        }
        return attribute.minValue;
    }
    
    @Override
    public String getAttributeMaxValue(String s, String s2) {
        IIOMetadataFormatImpl.Attribute attribute = this.getAttribute(s, s2);
        if (attribute.valueType != 2 && attribute.valueType != 6 && attribute.valueType != 10 && attribute.valueType != 14) {
            throw new IllegalArgumentException("Attribute not a range!");
        }
        return attribute.maxValue;
    }
    
    @Override
    public int getAttributeListMinLength(String s, String s2) {
        IIOMetadataFormatImpl.Attribute attribute = this.getAttribute(s, s2);
        if (attribute.valueType != 32) {
            throw new IllegalArgumentException("Attribute not a list!");
        }
        return attribute.listMinLength;
    }
    
    @Override
    public int getAttributeListMaxLength(String s, String s2) {
        IIOMetadataFormatImpl.Attribute attribute = this.getAttribute(s, s2);
        if (attribute.valueType != 32) {
            throw new IllegalArgumentException("Attribute not a list!");
        }
        return attribute.listMaxLength;
    }
    
    @Override
    public String getAttributeDescription(String str, String str2, Locale locale) {
        IIOMetadataFormatImpl.Element element = this.getElement(str);
        if (str2 == null) {
            throw new IllegalArgumentException("attrName == null!");
        }
        if ((IIOMetadataFormatImpl.Attribute)element.attrMap.get(str2) == null) {
            throw new IllegalArgumentException("No such attribute!");
        }
        return this.getResource(str + "/" + str2, locale);
    }
    
    private IIOMetadataFormatImpl.ObjectValue getObjectValue(String str) {
        IIOMetadataFormatImpl.ObjectValue objectValue = this.getElement(str).objectValue;
        if (objectValue == null) {
            throw new IllegalArgumentException("No object within element " + str + "!");
        }
        return objectValue;
    }
    
    @Override
    public int getObjectValueType(String s) {
        IIOMetadataFormatImpl.ObjectValue objectValue = this.getElement(s).objectValue;
        if (objectValue == null) {
            return 0;
        }
        return objectValue.valueType;
    }
    
    @Override
    public Class<?> getObjectClass(String s) {
        return this.getObjectValue(s).classType;
    }
    
    @Override
    public Object getObjectDefaultValue(String s) {
        return this.getObjectValue(s).defaultValue;
    }
    
    @Override
    public Object[] getObjectEnumerations(String s) {
        IIOMetadataFormatImpl.ObjectValue objectValue = this.getObjectValue(s);
        if (objectValue.valueType != 16) {
            throw new IllegalArgumentException("Not an enumeration!");
        }
        List<Object> enumeratedValues = objectValue.enumeratedValues;
        return enumeratedValues.toArray(new Object[enumeratedValues.size()]);
    }
    
    @Override
    public Comparable<?> getObjectMinValue(String s) {
        IIOMetadataFormatImpl.ObjectValue objectValue = this.getObjectValue(s);
        if ((objectValue.valueType & 0x2) != 0x2) {
            throw new IllegalArgumentException("Not a range!");
        }
        return objectValue.minValue;
    }
    
    @Override
    public Comparable<?> getObjectMaxValue(String s) {
        IIOMetadataFormatImpl.ObjectValue objectValue = this.getObjectValue(s);
        if ((objectValue.valueType & 0x2) != 0x2) {
            throw new IllegalArgumentException("Not a range!");
        }
        return objectValue.maxValue;
    }
    
    @Override
    public int getObjectArrayMinLength(String s) {
        IIOMetadataFormatImpl.ObjectValue objectValue = this.getObjectValue(s);
        if (objectValue.valueType != 32) {
            throw new IllegalArgumentException("Not a list!");
        }
        return objectValue.arrayMinLength;
    }
    
    @Override
    public int getObjectArrayMaxLength(String s) {
        IIOMetadataFormatImpl.ObjectValue objectValue = this.getObjectValue(s);
        if (objectValue.valueType != 32) {
            throw new IllegalArgumentException("Not a list!");
        }
        return objectValue.arrayMaxLength;
    }
    
    private static synchronized void createStandardFormat() {
        if (IIOMetadataFormatImpl.standardFormat == null) {
            IIOMetadataFormatImpl.standardFormat = new StandardMetadataFormat();
        }
    }
    
    public static IIOMetadataFormat getStandardFormatInstance() {
        createStandardFormat();
        return IIOMetadataFormatImpl.standardFormat;
    }
    
    static {
        IIOMetadataFormatImpl.standardFormat = null;
    }
    
    class Element
    {
        String elementName;
        int childPolicy;
        int minChildren;
        int maxChildren;
        List childList;
        List parentList;
        List attrList;
        Map attrMap;
        IIOMetadataFormatImpl.ObjectValue objectValue;
        
        Element() {
            this.minChildren = 0;
            this.maxChildren = 0;
            this.childList = new ArrayList();
            this.parentList = new ArrayList();
            this.attrList = new ArrayList();
            this.attrMap = new HashMap();
        }
    }
    
    class Attribute
    {
        String attrName;
        int valueType;
        int dataType;
        boolean required;
        String defaultValue;
        List enumeratedValues;
        String minValue;
        String maxValue;
        int listMinLength;
        int listMaxLength;
        
        Attribute() {
            this.valueType = 1;
            this.defaultValue = null;
        }
    }
    
    class ObjectValue
    {
        int valueType;
        Class classType;
        Object defaultValue;
        List enumeratedValues;
        Comparable minValue;
        Comparable maxValue;
        int arrayMinLength;
        int arrayMaxLength;
        
        ObjectValue() {
            this.valueType = 0;
            this.classType = null;
            this.defaultValue = null;
            this.enumeratedValues = null;
            this.minValue = null;
            this.maxValue = null;
            this.arrayMinLength = 0;
            this.arrayMaxLength = 0;
        }
    }
}
